@@ -42,6 +42,9 @@ mergeInto(LibraryManager.library, {
       } catch (e) {
         return callback(e);
       }
+      if (!req) {
+        return callback("Unable to connect to IndexedDB");
+      }
       req.onupgradeneeded = function(e) {
         var db = e.target.result;
         var transaction = e.target.transaction;
@@ -109,26 +112,30 @@ mergeInto(LibraryManager.library, {
       IDBFS.getDB(mount.mountpoint, function(err, db) {
         if (err) return callback(err);
 
-        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
-        transaction.onerror = function(e) {
-          callback(this.error);
-          e.preventDefault();
-        };
+        try {
+          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
+          transaction.onerror = function(e) {
+            callback(this.error);
+            e.preventDefault();
+          };
 
-        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
-        var index = store.index('timestamp');
+          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
+          var index = store.index('timestamp');
 
-        index.openKeyCursor().onsuccess = function(event) {
-          var cursor = event.target.result;
+          index.openKeyCursor().onsuccess = function(event) {
+            var cursor = event.target.result;
 
-          if (!cursor) {
-            return callback(null, { type: 'remote', db: db, entries: entries });
-          }
+            if (!cursor) {
+              return callback(null, { type: 'remote', db: db, entries: entries });
+            }
 
-          entries[cursor.primaryKey] = { timestamp: cursor.key };
+            entries[cursor.primaryKey] = { timestamp: cursor.key };
 
-          cursor.continue();
-        };
+            cursor.continue();
+          };
+        } catch (e) {
+          return callback(e);
+        }
       });
     },
     loadLocalEntry: function(path, callback) {
