@@ -59,15 +59,19 @@ var RuntimeGenerator = {
 
   // allocation on the top of memory, adjusted dynamically by sbrk
   dynamicAlloc: function(size) {
-    if (ASSERTIONS) size = '(assert(DYNAMICTOP > 0),' + size + ')'; // dynamic area must be ready
-#if USE_PTHREADS
-    if (typeof ENVIRONMENT_IS_PTHREAD !== 'undefined' && ENVIRONMENT_IS_PTHREAD) throw 'Runtime.dynamicAlloc is not available in pthreads!'; // This is because each worker has its own copy of DYNAMICTOP, of which main thread is authoritative.
-#endif
-    var ret = RuntimeGenerator.alloc(size, 'DYNAMIC');
-    if (SAFE_HEAP) ret += '; if (asm) { Runtime.setDynamicTop(DYNAMICTOP); }';
-    ret += '; if (DYNAMICTOP >= TOTAL_MEMORY) { var success = enlargeMemory(); if (!success) { DYNAMICTOP = ret; ';
-    if (SAFE_HEAP) ret += 'if (asm) { Runtime.setDynamicTop(DYNAMICTOP); }';
-    ret += ' return 0; } }'
+    var ret = '';
+    if (ASSERTIONS) ret += 'assert(DYNAMICTOP_PTR);'; // dynamic area must be ready
+    ret += 'var ret = HEAP32[DYNAMICTOP_PTR>>2];'
+      + 'var end = (((ret + size + 15)|0) & -16);'
+      + 'HEAP32[DYNAMICTOP_PTR>>2] = end;'
+      + 'if (end >= TOTAL_MEMORY) {'
+      +   'var success = enlargeMemory();'
+      +     'if (!success) {'
+      +       'HEAP32[DYNAMICTOP_PTR>>2] = ret;'
+      +       'return 0;'
+      +    '}'
+      +  '}'
+      + 'return ret;';
     return ret;
   },
 
@@ -100,8 +104,12 @@ var RuntimeGenerator = {
   }
 };
 
-function unInline(name_, params) {
-  var src = '(function(' + params + ') { var ret = ' + RuntimeGenerator[name_].apply(null, params) + '; return ret; })';
+function unInline(name_, params, isExpression) {
+  if (isExpression) {
+    var src = '(function(' + params + ') { var ret = ' + RuntimeGenerator[name_].apply(null, params) + '; return ret; })';
+  } else {
+    var src = '(function(' + params + ') { ' + RuntimeGenerator[name_].apply(null, params) + '})';
+  }
   var ret = eval(src);
   return ret;
 }
@@ -124,6 +132,7 @@ var Runtime = {
   // word is placed in tempRet0. This provides an accessor for that value.
   setTempRet0: function(value) {
     tempRet0 = value;
+    return value;
   },
   getTempRet0: function() {
     return tempRet0;
@@ -196,12 +205,10 @@ var Runtime = {
 #if ASSERTIONS
       assert(args.length == sig.length-1);
 #endif
-      if (!args.splice) args = Array.prototype.slice.call(args);
-      args.splice(0, 0, ptr);
 #if ASSERTIONS
       assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
 #endif
-      return Module['dynCall_' + sig].apply(null, args);
+      return Module['dynCall_' + sig].apply(null, [ptr].concat(args));
     } else {
 #if ASSERTIONS
       assert(sig.length == 1);
@@ -274,6 +281,14 @@ var Runtime = {
       }
     }
     throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
+#else
+#if BINARYEN
+    // we can simply appent to the wasm table
+    var table = Module['wasmTable'];
+    var ret = table.length;
+    table.grow(1);
+    table.set(ret, func);
+    return ret;
 #else
     Runtime.alignFunctionTables(); // XXX we should rely on this being an invariant
     var tables = Runtime.getFunctionTables();
@@ -285,6 +300,7 @@ var Runtime = {
       table.push(func);
     }
     return ret;
+#endif
 #endif
   },
 
@@ -304,12 +320,24 @@ var Runtime = {
   loadedDynamicLibraries: [],
 
   loadDynamicLibrary: function(lib) {
-    // TODO: addRunDep etc., do asynchronously when in the browser. for now we assume we can do a sync xhr, no mem init files in libs, and we ignore the sync xhr lag
+    var libModule;
+#if BINARYEN
+    var bin;
+    if (lib.buffer) {
+      // we were provided the binary, in a typed array
+      bin = lib;
+    } else {
+      // load the binary synchronously
+      bin = Module['readBinary'](lib);
+    }
+    libModule = Runtime.loadWebAssemblyModule(bin);
+#else
     var src = Module['read'](lib);
-    var libModule = eval(src)(
+    libModule = eval(src)(
       Runtime.alignFunctionTables(),
       Module
     );
+#endif
     // add symbols into global namespace TODO: weak linking etc.
     for (var sym in libModule) {
       if (!Module.hasOwnProperty(sym)) {
@@ -327,6 +355,112 @@ var Runtime = {
     }
     Runtime.loadedDynamicLibraries.push(libModule);
   },
+
+#if BINARYEN
+  // Loads a side module from binary data
+  loadWebAssemblyModule: function(binary) {
+    var int32View = new Uint32Array(new Uint8Array(binary.subarray(0, 24)).buffer);
+    assert(int32View[0] == 0x6d736100, 'need to see wasm magic number'); // \0wasm
+    // we should see the dylink section right after the magic number and wasm version
+    assert(binary[8] === 0, 'need the dylink section to be first')
+    var next = 9;
+    function getLEB() {
+      var ret = 0;
+      var mul = 1;
+      while (1) {
+        var byte = binary[next++];
+        ret += ((byte & 0x7f) * mul);
+        mul *= 0x80;
+        if (!(byte & 0x80)) break;
+      }
+      return ret;
+    }
+    var sectionSize = getLEB();
+    assert(binary[next] === 6);                 next++; // size of "dylink" string
+    assert(binary[next] === 'd'.charCodeAt(0)); next++;
+    assert(binary[next] === 'y'.charCodeAt(0)); next++;
+    assert(binary[next] === 'l'.charCodeAt(0)); next++;
+    assert(binary[next] === 'i'.charCodeAt(0)); next++;
+    assert(binary[next] === 'n'.charCodeAt(0)); next++;
+    assert(binary[next] === 'k'.charCodeAt(0)); next++;
+    var memorySize = getLEB();
+    var tableSize = getLEB();
+    var env = Module['asmLibraryArg'];
+    // TODO: use only memoryBase and tableBase, need to update asm.js backend
+    var table = Module['wasmTable'];
+    var oldTableSize = table.length;
+    env['memoryBase'] = env['gb'] = Runtime.alignMemory(getMemory(memorySize + Runtime.STACK_ALIGN), Runtime.STACK_ALIGN); // TODO: add to cleanups
+    env['tableBase'] = env['fb'] = oldTableSize;
+    var originalTable = table;
+    table.grow(tableSize);
+    assert(table === originalTable);
+    // zero-initialize memory and table TODO: in some cases we can tell it is already zero initialized
+    for (var i = env['memoryBase']; i < env['memoryBase'] + memorySize; i++) {
+      HEAP8[i] = 0;
+    }
+    for (var i = env['tableBase']; i < env['tableBase'] + tableSize; i++) {
+      table.set(i, null);
+    }
+    // copy currently exported symbols so the new module can import them
+    for (var x in Module) {
+      if (!(x in env)) {
+        env[x] = Module[x];
+      }
+    }
+    var info = {
+      global: {
+        'NaN': NaN,
+        'Infinity': Infinity,
+        'Math': Math
+      },
+      env: env
+    };
+#if ASSERTIONS
+    var oldTable = [];
+    for (var i = 0; i < oldTableSize; i++) {
+      oldTable.push(table.get(i));
+    }
+#endif
+    // create a module from the instance
+    var instance = new WebAssembly.Instance(new WebAssembly.Module(binary), info);
+#if ASSERTIONS
+    // the table should be unchanged
+    assert(table === originalTable);
+    assert(table === Module['wasmTable']);
+    if (instance.exports['table']) {
+      assert(table === instance.exports['table']);
+    }
+    // the old part of the table should be unchanged
+    for (var i = 0; i < oldTableSize; i++) {
+      assert(table.get(i) === oldTable[i], 'old table entries must remain the same');
+    }
+    // verify that the new table region was filled in
+    for (var i = 0; i < tableSize; i++) {
+      assert(table.get(oldTableSize + i) !== undefined, 'table entry was not filled in');
+    }
+#endif
+    var exports = {};
+    for (var e in instance.exports) {
+      var value = instance.exports[e];
+      if (typeof value === 'number') {
+        // relocate it - modules export the absolute value, they can't relocate before they export
+        value = value + env['memoryBase'];
+      }
+      exports[e] = value;
+    }
+    // initialize the module
+    var init = exports['__post_instantiate'];
+    if (init) {
+      if (runtimeInitialized) {
+        init();
+      } else {
+        // we aren't ready to run compiled code yet
+        __ATINIT__.push(init);
+      }
+    }
+    return exports;
+  },
+#endif
 #endif
 
   warnOnce: function(text) {
@@ -340,15 +474,28 @@ var Runtime = {
   funcWrappers: {},
 
   getFuncWrapper: function(func, sig) {
+    if (!func) return; // on null pointer, return undefined
     assert(sig);
     if (!Runtime.funcWrappers[sig]) {
       Runtime.funcWrappers[sig] = {};
     }
     var sigCache = Runtime.funcWrappers[sig];
     if (!sigCache[func]) {
-      sigCache[func] = function dynCall_wrapper() {
-        return Runtime.dynCall(sig, func, arguments);
-      };
+      // optimize away arguments usage in common cases
+      if (sig.length === 1) {
+        sigCache[func] = function dynCall_wrapper() {
+          return Runtime.dynCall(sig, func);
+        };
+      } else if (sig.length === 2) {
+        sigCache[func] = function dynCall_wrapper(arg) {
+          return Runtime.dynCall(sig, func, [arg]);
+        };
+      } else {
+        // general case
+        sigCache[func] = function dynCall_wrapper() {
+          return Runtime.dynCall(sig, func, Array.prototype.slice.call(arguments));
+        };
+      }
     }
     return sigCache[func];
   },
@@ -419,11 +566,11 @@ var Runtime = {
 #endif
 };
 
-Runtime.stackAlloc = unInline('stackAlloc', ['size']);
-Runtime.staticAlloc = unInline('staticAlloc', ['size']);
-Runtime.dynamicAlloc = unInline('dynamicAlloc', ['size']);
-Runtime.alignMemory = unInline('alignMemory', ['size', 'quantum']);
-Runtime.makeBigInt = unInline('makeBigInt', ['low', 'high', 'unsigned']);
+Runtime.stackAlloc = unInline('stackAlloc', ['size'], true);
+Runtime.staticAlloc = unInline('staticAlloc', ['size'], true);
+Runtime.dynamicAlloc = unInline('dynamicAlloc', ['size'], false);
+Runtime.alignMemory = unInline('alignMemory', ['size', 'quantum'], true);
+Runtime.makeBigInt = unInline('makeBigInt', ['low', 'high', 'unsigned'], true);
 
 if (MAIN_MODULE || SIDE_MODULE) {
   Runtime.tempRet0 = 0;
@@ -432,6 +579,7 @@ if (MAIN_MODULE || SIDE_MODULE) {
   };
   Runtime.setTempRet0 = function(x) {
     Runtime.tempRet0 = x;
+    return x;
   };
 }
 
