@@ -1,5 +1,6 @@
 #include <strings.h>
 #include <ctype.h>
+#include "libc.h"
 
 int strcasecmp(const char *_l, const char *_r)
 {
@@ -7,3 +8,10 @@ int strcasecmp(const char *_l, const char *_r)
 	for (; *l && *r && (*l == *r || tolower(*l) == tolower(*r)); l++, r++);
 	return tolower(*l) - tolower(*r);
 }
+
+int __strcasecmp_l(const char *l, const char *r, locale_t loc)
+{
+	return strcasecmp(l, r);
+}
+
+weak_alias(__strcasecmp_l, strcasecmp_l);
