@@ -70,130 +70,6 @@ double _pthread_msecs_until(const struct timespec *restrict at)
 	return msecs;
 }
 
-#if 0
-int pthread_mutex_lock(pthread_mutex_t *mutex)
-{
-	if (!mutex) return EINVAL;
-	assert(pthread_self() != 0);
-	assert(pthread_self()->tid != 0);
-
-	if (mutex->_m_lock == pthread_self()->tid) {
-		if ((mutex->_m_type&3) == PTHREAD_MUTEX_RECURSIVE) {
-			if ((unsigned)mutex->_m_count >= INT_MAX) return EAGAIN;
-			++mutex->_m_count;
-			return 0;
-		} else if ((mutex->_m_type&3) == PTHREAD_MUTEX_ERRORCHECK) {
-			return EDEADLK;
-		}
-	}
-
-	int threadCancelType = _pthread_getcanceltype();
-
-	int c = emscripten_atomic_cas_u32(&mutex->_m_addr, 0, 1);
-	if (c != 0) {
-		do {
-			if (c == 2 || emscripten_atomic_cas_u32(&mutex->_m_addr, 1, 2) != 0) {
-				double msecs = INFINITY;
-				if (threadCancelType == PTHREAD_CANCEL_ASYNCHRONOUS) {
-					// Sleep in small slices so that we can test cancellation to honor PTHREAD_CANCEL_ASYNCHRONOUS.
-					__pthread_testcancel();
-					msecs = 100;
-				}
-				emscripten_futex_wait(&mutex->_m_addr, 2, msecs);
-			}
-		} while((c = emscripten_atomic_cas_u32(&mutex->_m_addr, 0, 2)));
-	}
-
-	__pthread_mutex_locked(mutex);
-	return 0;
-}
-
-int pthread_mutex_unlock(pthread_mutex_t *mutex)
-{
-	if (!mutex) return EINVAL;
-	assert(pthread_self() != 0);
-
-	if (mutex->_m_type != PTHREAD_MUTEX_NORMAL) {
-		if (mutex->_m_lock != pthread_self()->tid) return EPERM;
-		if ((mutex->_m_type&3) == PTHREAD_MUTEX_RECURSIVE && mutex->_m_count) {
-			--mutex->_m_count;
-			return 0;
-		}
-	}
-
-	mutex->_m_lock = 0;
-	if (emscripten_atomic_sub_u32((uint32_t*)&mutex->_m_addr, 1) != 1)
-	{
-		emscripten_atomic_store_u32((uint32_t*)&mutex->_m_addr, 0);
-		emscripten_futex_wake((uint32_t*)&mutex->_m_addr, 1);
-	}
-	return 0;
-}
-
-int pthread_mutex_trylock(pthread_mutex_t *mutex)
-{
-	if (!mutex) return EINVAL;
-	if (mutex->_m_lock == pthread_self()->tid) {
-		if ((mutex->_m_type&3) == PTHREAD_MUTEX_RECURSIVE) {
-			if ((unsigned)mutex->_m_count >= INT_MAX) return EAGAIN;
-			++mutex->_m_count;
-			return 0;
-		} else if ((mutex->_m_type&3) == PTHREAD_MUTEX_ERRORCHECK) {
-			return EDEADLK;
-		}
-	}
-
-	if (emscripten_atomic_cas_u32(&mutex->_m_addr, 0, 1) == 0) {
-		__pthread_mutex_locked(mutex);
-		return 0;
-	}
-	else
-		return EBUSY;
-}
-
-int pthread_mutex_timedlock(pthread_mutex_t *restrict mutex, const struct timespec *restrict at)
-{
-	if (!mutex || !at) return EINVAL;
-	if (mutex->_m_lock == pthread_self()->tid) {
-		if ((mutex->_m_type&3) == PTHREAD_MUTEX_RECURSIVE) {
-			if ((unsigned)mutex->_m_count >= INT_MAX) return EAGAIN;
-			++mutex->_m_count;
-			return 0;
-		} else if ((mutex->_m_type&3) == PTHREAD_MUTEX_ERRORCHECK) {
-			return EDEADLK;
-		}
-	}
-
-	int threadCancelType = _pthread_getcanceltype();
-	int c = emscripten_atomic_cas_u32(&mutex->_m_addr, 0, 1);
-	if (c != 0) {
-		do {
-			if (c == 2 || emscripten_atomic_cas_u32(&mutex->_m_addr, 1, 2) != 0)
-			{
-				if (at->tv_nsec < 0 || at->tv_nsec >= 1000000000) return EINVAL;
-				double msecs = _pthread_msecs_until(at);
-				if (msecs <= 0) return ETIMEDOUT;
-
-				// Sleep in small slices if thread type is PTHREAD_CANCEL_ASYNCHRONOUS
-				// so that we can honor PTHREAD_CANCEL_ASYNCHRONOUS requests.
-				if (threadCancelType == PTHREAD_CANCEL_ASYNCHRONOUS) {
-					__pthread_testcancel();
-					if (msecs > 100) msecs = 100;
-				}
-				int ret = emscripten_futex_wait(&mutex->_m_addr, 2, msecs);
-				if (ret == 0) break;
-				else if (threadCancelType != PTHREAD_CANCEL_ASYNCHRONOUS || _pthread_msecs_until(at) <= 0) {
-					return ETIMEDOUT;
-				}
-			}
-		} while((c = emscripten_atomic_cas_u32(&mutex->_m_addr, 0, 2)));
-	}
-
-	__pthread_mutex_locked(mutex);
-	return 0;
-}
-#endif
-
 int sched_get_priority_max(int policy)
 {
 	// Web workers do not actually support prioritizing threads,
@@ -235,7 +111,7 @@ void __pthread_testcancel()
 	struct pthread *self = pthread_self();
 	if (self->canceldisable) return;
 	if (_pthread_isduecanceled(self)) {
-		EM_ASM( throw 'Canceled!'; );
+		EM_ASM(throw 'Canceled!');
 	}
 }
 
@@ -291,8 +167,12 @@ int usleep(unsigned usec)
 static em_queued_call *em_queued_call_malloc()
 {
 	em_queued_call *call = (em_queued_call*)malloc(sizeof(em_queued_call));
-	call->operationDone = 0;
-	call->functionPtr = 0;
+	assert(call); // Not a programming error, but use assert() in debug builds to catch OOM scenarios.
+	if (call)
+	{
+		call->operationDone = 0;
+		call->functionPtr = 0;
+	}
 	return call;
 }
 static void em_queued_call_free(em_queued_call *call)
@@ -309,19 +189,6 @@ static void _do_call(em_queued_call *q)
 {
 	switch(q->functionEnum)
 	{
-		case EM_PROXIED_UTIME: q->returnValue.i = utime(q->args[0].cp, (struct utimbuf*)q->args[1].vp); break;
-		case EM_PROXIED_UTIMES: q->returnValue.i = utimes(q->args[0].cp, (struct timeval*)q->args[1].vp); break;
-		case EM_PROXIED_CHROOT: q->returnValue.i = chroot(q->args[0].cp); break;
-		case EM_PROXIED_FPATHCONF: q->returnValue.i = fpathconf(q->args[0].i, q->args[1].i); break;
-		case EM_PROXIED_CONFSTR: q->returnValue.i = confstr(q->args[0].i, q->args[1].cp, q->args[2].i); break;
-		case EM_PROXIED_SYSCONF: q->returnValue.i = sysconf(q->args[0].i); break;
-		case EM_PROXIED_ATEXIT: q->returnValue.i = atexit(q->args[0].vp); break;
-		case EM_PROXIED_GETENV: q->returnValue.cp = getenv(q->args[0].cp); break;
-		case EM_PROXIED_CLEARENV: q->returnValue.i = clearenv(); break;
-		case EM_PROXIED_SETENV: q->returnValue.i = setenv(q->args[0].cp, q->args[1].cp, q->args[2].i); break;
-		case EM_PROXIED_UNSETENV: q->returnValue.i = unsetenv(q->args[0].cp); break;
-		case EM_PROXIED_PUTENV: q->returnValue.i = putenv(q->args[0].cp); break;
-		case EM_PROXIED_TZSET: tzset(); break;
 		case EM_PROXIED_PTHREAD_CREATE: q->returnValue.i = pthread_create(q->args[0].vp, q->args[1].vp, q->args[2].vp, q->args[3].vp); break;
 		case EM_PROXIED_SYSCALL: q->returnValue.i = emscripten_syscall(q->args[0].i, q->args[1].vp); break;
 		case EM_FUNC_SIG_V: ((em_func_v)q->functionPtr)(); break;
@@ -578,10 +445,11 @@ void EMSCRIPTEN_KEEPALIVE emscripten_main_thread_process_queued_calls()
 
 int emscripten_sync_run_in_main_runtime_thread_(EM_FUNC_SIGNATURE sig, void *func_ptr, ...)
 {
-	va_list args;
-	va_start(args, func_ptr);
 	int numArguments = EM_FUNC_SIG_NUM_FUNC_ARGUMENTS(sig);
 	em_queued_call q = { sig, func_ptr };
+
+	va_list args;
+	va_start(args, func_ptr);
 	for(int i = 0; i < numArguments; ++i)
 		q.args[i].i = va_arg(args, int);
 	va_end(args);
@@ -591,12 +459,14 @@ int emscripten_sync_run_in_main_runtime_thread_(EM_FUNC_SIGNATURE sig, void *fun
 
 void emscripten_async_run_in_main_runtime_thread_(EM_FUNC_SIGNATURE sig, void *func_ptr, ...)
 {
-	va_list args;
-	va_start(args, func_ptr);
 	int numArguments = EM_FUNC_SIG_NUM_FUNC_ARGUMENTS(sig);
 	em_queued_call *q = em_queued_call_malloc();
+	if (!q) return;
 	q->functionEnum = sig;
 	q->functionPtr = func_ptr;
+
+	va_list args;
+	va_start(args, func_ptr);
 	for(int i = 0; i < numArguments; ++i)
 		q->args[i].i = va_arg(args, int);
 	va_end(args);
@@ -608,12 +478,14 @@ void emscripten_async_run_in_main_runtime_thread_(EM_FUNC_SIGNATURE sig, void *f
 
 em_queued_call *emscripten_async_waitable_run_in_main_runtime_thread_(EM_FUNC_SIGNATURE sig, void *func_ptr, ...)
 {
-	va_list args;
-	va_start(args, func_ptr);
 	int numArguments = EM_FUNC_SIG_NUM_FUNC_ARGUMENTS(sig);
 	em_queued_call *q = em_queued_call_malloc();
+	if (!q) return;
 	q->functionEnum = sig;
 	q->functionPtr = func_ptr;
+
+	va_list args;
+	va_start(args, func_ptr);
 	for(int i = 0; i < numArguments; ++i)
 		q->args[i].i = va_arg(args, int);
 	va_end(args);
@@ -641,16 +513,6 @@ float EMSCRIPTEN_KEEPALIVE emscripten_atomic_load_f32(const void *addr)
 #define NUM_64BIT_LOCKS 256
 static int emulated64BitAtomicsLocks[NUM_64BIT_LOCKS] = {};
 
-uint32_t EMSCRIPTEN_KEEPALIVE emscripten_atomic_exchange_u32(void/*uint32_t*/ *addr, uint32_t newVal)
-{
-	uint32_t oldVal, oldVal2;
-	do {
-		oldVal = emscripten_atomic_load_u32(addr);
-		oldVal2 = emscripten_atomic_cas_u32(addr, oldVal, newVal);
-	} while (oldVal != oldVal2);
-	return oldVal;
-}
-
 #define SPINLOCK_ACQUIRE(addr) do { while(emscripten_atomic_exchange_u32((void*)(addr), 1)) /*nop*/; } while(0)
 #define SPINLOCK_RELEASE(addr) emscripten_atomic_store_u32((void*)(addr), 0)
 
@@ -888,4 +750,61 @@ uint64_t __atomic_fetch_xor_8(void *ptr, uint64_t value, int memmodel)
 	return _emscripten_atomic_fetch_and_xor_u64(ptr, value);
 }
 
+typedef struct main_args
+{
+  int argc;
+  char **argv;
+} main_args;
+
+extern int __call_main(int argc, char **argv);
+
+void *__emscripten_thread_main(void *param)
+{
+  emscripten_set_thread_name(pthread_self(), "Application main thread"); // This is the main runtime thread for the application.
+  main_args *args = (main_args*)param;
+  return (void*)__call_main(args->argc, args->argv);
+}
+
+static volatile main_args _main_arguments;
+
+// TODO: Create a separate library of this to be able to drop EMSCRIPTEN_KEEPALIVE from this definition.
+int EMSCRIPTEN_KEEPALIVE proxy_main(int argc, char **argv)
+{
+  if (emscripten_has_threading_support())
+  {
+    pthread_attr_t attr;
+    pthread_attr_init(&attr);
+    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
+
+    // TODO: Read this from -s STACK_SIZE parameter, and make actual main browser thread stack something tiny, or create a -s PROXY_THREAD_STACK_SIZE parameter.
+#define EMSCRIPTEN_PTHREAD_STACK_SIZE (128*1024)
+
+    pthread_attr_setstacksize(&attr, (EMSCRIPTEN_PTHREAD_STACK_SIZE));
+    // TODO: Add a -s PROXY_CANVASES_TO_THREAD=parameter or similar to allow configuring this
+#ifdef EMSCRIPTEN_PTHREAD_TRANSFERRED_CANVASES
+    // If user has defined EMSCRIPTEN_PTHREAD_TRANSFERRED_CANVASES, then transfer those canvases over to the pthread.
+    emscripten_pthread_attr_settransferredcanvases(&attr, (EMSCRIPTEN_PTHREAD_TRANSFERRED_CANVASES));
+#else
+    // Otherwise by default, transfer whatever is set to Module.canvas.
+    if (EM_ASM_INT_V({ return !!(Module['canvas']); })) emscripten_pthread_attr_settransferredcanvases(&attr, "#canvas");
+#endif
+    _main_arguments.argc = argc;
+    _main_arguments.argv = argv;
+    pthread_t thread;
+    int rc = pthread_create(&thread, &attr, __emscripten_thread_main, (void*)&_main_arguments);
+    pthread_attr_destroy(&attr);
+    if (rc)
+    {
+      // Proceed by running main() on the main browser thread as a fallback.
+      return __call_main(_main_arguments.argc, _main_arguments.argv);
+    }
+    EM_ASM(Module['noExitRuntime'] = true);
+    return 0;
+  }
+  else
+  {
+    return __call_main(_main_arguments.argc, _main_arguments.argv);
+  }
+}
+
 weak_alias(__pthread_testcancel, pthread_testcancel);
